using System;
using System.Text;
using System.Windows.Forms;

using CookComputing.XmlRpc;

namespace GameControler
{
	public struct SStatus
	{
		public int Code;
		public string Name;
		public void SetDefault() {
			Code = -1;
			Name = "???";
		}
	}

	public struct SVersion
	{
		public string Name;
		public string Version;
		[XmlRpcMissingMappingAttribute(MappingAction.Ignore)]	// Build is a recent addition
		public string Build;
		public void SetDefault() {
			Version = "???";
			Name = "???";
			Build = "";
		}
	}

	public struct SStartServerParamInternet
	{
		public string Login;
		public string Password;
		public void SetDefault() {
			Login = "";
			Password = "";
		}
	}

	public struct SServerOptions
	{
		public string Name;
		public string Comment;
		public string Password;
		public string PasswordForSpectator;
		public int NextMaxPlayers;
		public int NextMaxSpectators;
		public int NextVehicleNetQuality;
		public int NextLadderMode;	// 0 = NoLadder, 1=EnabledLadder, 2=ForcedLadder
		public int CurrentMaxPlayers;		// only used for get
		public int CurrentMaxSpectators;	// only used for get
		public int CurrentLadderMode;		// only used for get
		public int CurrentVehicleNetQuality;// only used for get
		public bool IsP2PUpload;
		public bool IsP2PDownload;
		public int CurrentCallVoteTimeOut;
		public int NextCallVoteTimeOut;
		public double CallVoteRatio;
		public bool AllowChallengeDownload;
		
		public void SetDefault() 
		{
			Comment = Name = Password = PasswordForSpectator = "";
			CurrentMaxPlayers = NextMaxPlayers = 1;
			CurrentMaxSpectators = NextMaxSpectators = 1;
			CurrentLadderMode = NextLadderMode = 1;
			CurrentVehicleNetQuality = NextVehicleNetQuality = 0;
			IsP2PUpload = IsP2PDownload = false;
			CurrentCallVoteTimeOut = NextCallVoteTimeOut = 60000;
			CallVoteRatio = 0.5;
			AllowChallengeDownload = true;
		}
	}

	public struct SGameInfo
	{
		public int GameMode;
		public int ChatTime;
		public int NbChallenge;			// only used for GetGameInfo
		public int RoundsPointsLimit;
		public bool RoundsUseNewRules;
		public int TimeAttackLimit;
		public int TimeAttackSynchStartPeriod;
		public int TeamPointsLimit;
		public int TeamMaxPoints;
		public bool TeamUseNewRules;
		public int LapsNbLaps;
		public int LapsTimeLimit;
		
		public void SetDefault() {
			ChatTime = GameMode = LapsNbLaps = LapsTimeLimit 
				= NbChallenge = RoundsPointsLimit = TeamMaxPoints
				= TeamPointsLimit = TimeAttackLimit = 0;
			TimeAttackSynchStartPeriod = 0;
			NbChallenge = 0;
			RoundsUseNewRules = TeamUseNewRules = false;
		}
	}

	public struct SFullGameInfo
	{
		public SGameInfo CurrentGameInfos;
		public SGameInfo NextGameInfos;
		public void SetDefault() {
			CurrentGameInfos.SetDefault();
			NextGameInfos.SetDefault();
		}
	}

	public struct SChallengeInfo
	{
		public string Name;
		public string FileName;
		public string Author;
		public string Environnement;
		[XmlRpcMissingMappingAttribute(MappingAction.Ignore)]	// Mood was called "DayTime" before.
		public string Mood;
		public int BronzeTime;
		public int SilverTime;
		public int GoldTime;
		[XmlRpcMissingMappingAttribute(MappingAction.Ignore)]	// wasn't there before.
		public int AuthorTime;
		public int CopperPrice;
		public bool LapRace;
	}

	public struct SChallengeQuickInfo
	{
		public string Name;
		public string FileName;
		public string Environnement;
		[XmlRpcMissingMappingAttribute(MappingAction.Ignore)]	// Mood was called "DayTime" before.
		public string Mood;
		public int GoldTime;
		public int CopperPrice;
	}

	public struct SLadderStats
	{
		public int Ranking;
		public double Score;
		public double LastMatchScore;
		public int NbrMatchWins;
		public int NbrMatchDraws;
		public int NbrMatchLosses;
		public int NbrTotalPlayers;
		public string TeamName;
		public int TeamRanking;
		public int NbrTotalTeams;
	}

	public struct SPackDescInfo
	{
		public string FileName;
		public int Checksum;
	}

	public struct SSkinInfo
	{
		public string Environnement;
		public SPackDescInfo PackDesc;
	}

	public struct SPlayerInfo
	{
		public string Login;
		public string NickName;
		public string IPAddress;
		public string ConnectionType;
		[XmlRpcMissingMappingAttribute(MappingAction.Ignore)]
		public string Nation;
		public int PlayerId;
		public bool IsSpectator;
		public bool IsInOfficialMode;
		public bool HasJoinMatch;
		public SPackDescInfo Avatar;
		public SSkinInfo[] Skins;
		public SLadderStats LadderStats;
	}

	public struct SPlayerQuickInfo
	{
		public string Login;
		public string NickName;
		public int PlayerId;
		public int TeamId;
		public bool IsSpectator;
		public bool IsInOfficialMode;
		public int LadderRanking;	
	}

	public struct SPlayerInfoBanned
	{
		public string Login;
		public string ClientName;
		public string IPAddress;
	}

	public struct SPlayerRanking
	{
		public string Login;
		public string NickName;
		[XmlRpcMissingMappingAttribute(MappingAction.Ignore)]
		public int PlayerId;
		public int Rank;
		public int BestTime;
		public int Score;
		public int NbrLapsFinished;
		[XmlRpcMissingMappingAttribute(MappingAction.Ignore)]
		public int[] BestChekpoints;
		public double LadderScore;
	}

	public struct SPlayerNetworkInfo
	{
		public string Login;
		public string IPAddress;
		public int LastTransferTime;
		public int DeltaBetweenTwoLastNetState;
		public double PacketLossRate;
	}

	public struct SNetworkStats
	{
		public int Uptime;
		public int NbrConnection;
		public int MeanConnectionTime;
		public int MeanNbrPlayer;
		public int RecvNetRate;
		public int SendNetRate;
		public int TotalReceivingSize;
		public int TotalSendingSize;
		public SPlayerNetworkInfo[] PlayerNetInfos;
	}

	public struct SPlayerInfoBlack
	{
		public string Login;
	}

	public struct SMessageAnswer
	{
		public string Login;
		public int 	  PlayerId;
		public int 	  Answer;
	}

	public struct SValuePair
	{
		public int CurrentValue;
		public int NextValue;
	}
}

namespace GameControler.XmlRpcInterface
{
	[XmlRpcUrl("gbx://127.0.0.1:5000")]
	public interface GameboxRemote
	{
		// == AUTHENTICATE ======================================

		// Allow user authentication by specifying a login and a password, to gain access to the set of functionalities corresponding to this authorization level.
		[XmlRpcMethod] bool Authenticate(string _Login, string _Password);

		// Change the password for the specified login/user. Only available to SuperAdmin.
		[XmlRpcMethod] bool ChangeAuthPassword(string _Login, string _NewPassword);


		// == MESSAGES ======================================

		// Returns a struct with the Name and Version of the application remotely controled.
		[XmlRpcMethod] SVersion GetVersion();

		// Call vote for cmd. The command is a XML string corresponding to an XmlRpc request. Only available to Admin.
		[XmlRpcMethod] bool CallVote(string _Message);

		// Cancel the current vote. Only available to Admin.
		[XmlRpcMethod] bool CancelVote();

		// Add a new command 'cmd' for the chat, i.e. a chat message that begins with '/cmd ' will not be transmitted to clients. Only available to Admin.
		[XmlRpcMethod] bool AddChatCommand(string _Cmd);

		// Remove a chat command. Only available to Admin.
		[XmlRpcMethod] bool RemoveChatCommand(string _Cmd);

		// Remove all chat commands. Only available to Admin.
		[XmlRpcMethod] bool CleanChatCommand();

		// Returns the list of chat commands. This method take two parameters. The first parameter specifies the maximum number of infos to be returned, and the second one the starting index in the list. The list is an array of structure. Each structure contains the following fields : Name.
		[XmlRpcMethod] string[] GetChatCommandList(int _Number, int _FirstIndex);

		// Send a text message to all clients. Only available to Admin.
		[XmlRpcMethod] bool ChatSend(string _Message);

		// Send a text message to all clients without the server login. Only available to Admin.
		[XmlRpcMethod] bool ChatSendServerMessage(string _Message);

		// Send a text message to a client without the server login. Only available to Admin.
		[XmlRpcMethod] bool ChatSendServerMessageToId(string _Message, int _Id);

		// Send a text message to a client without the server login. Only available to Admin.
		[XmlRpcMethod] bool ChatSendServerMessageToLogin(string _Message, string _Login);

		// Send a text message to the client with the specified login. Only available to Admin.
		[XmlRpcMethod] bool ChatSendToLogin(string _Message, string _Login);

		// Send a text message to with the specified PlayerId. Only available to Admin.
		[XmlRpcMethod] bool ChatSendToId(string _Message, int _Id);

		// Returns the last chat lines. Maximum of 20 lines. Only available to Admin.
		[XmlRpcMethod] string[] GetChatLines();

		// Display a text message on all clients. The parameters are the text message to display, the label of the first button, the label of the second button, a timeout for the message box. This message is displayed in a message box with one or two buttons, if the label are not empty. The message box will disapear after the timeout if it's non-zero. Only available to Admin.
		[XmlRpcMethod] bool SendDisplayServerMessage(string _Text, string _Button1, string _Button2, int _TimeOut);

		// Display a text message on the client with the specified UId. The parameters are the UId of the player, the text message to display, the label of the first button, the label of the second button, and a timeout for the message box. This message is displayed in a message box with one or two buttons, if the corresponding label are not empty. The message box will disapear after the timeout if it's non-zero. Only available to Admin.
		[XmlRpcMethod] bool SendDisplayServerMessageToId(int _PlayerId, string _Text, string _Button1, string _Button2, int _TimeOut);

		// Display a text message on the client with the specified login. The parameters are the login of the player, the text message to display, the label of the first button, the label of the second button, and a timeout for the message box. This message is displayed in a message box with one or two buttons, if the corresponding label are not empty. The message box will disapear after the timeout if it's non-zero. Only available to Admin.
		[XmlRpcMethod] bool SendDisplayServerMessageToLogin(string _Login, string _Text, string _Button1, string _Button2, int _TimeOut);

		// Hide the displayed message on all clients. Only available to Admin.
		[XmlRpcMethod] bool SendHideServerMessage();

		// Hide the displayed message on the client with the specified UId. Only available to Admin.
		[XmlRpcMethod] bool SendHideServerMessageToId(int _PlayerId);

		// Hide the displayed message on the client with the specified login. Only available to Admin.
		[XmlRpcMethod] bool SendHideServerMessageToLogin(string _Login);

		// Returns the results of the message boxes, as an array of structs {login, playerid, result} result==0 -> no answer, result==1 or 2 -> answer from the player. Only available to Admin.
		[XmlRpcMethod] SMessageAnswer[] GetServerMessageResults();

		// == KICK / BAN ======================================
		
		// Kick the player with the specified login. Only available to Admin.
		[XmlRpcMethod] bool Kick(string _Login);

		// Kick the player with the specified PlayerId. Only available to Admin.
		[XmlRpcMethod] bool KickId(int _Id);

		// Ban the player with the specified login. Only available to Admin. 
		[XmlRpcMethod] bool Ban(string _Login);

		// Ban the player with the specified PlayerId. Only available to Admin.
		[XmlRpcMethod] bool BanId(int _Id);

		// Unban the player with the specified client name. Only available to Admin.
		[XmlRpcMethod] bool UnBan(string _Login);

		// Clean the ban list of the server. Only available to Admin.
		[XmlRpcMethod] bool CleanBanList();

		// Returns the list of player banned. This method take two parameters. The first parameter specifies the maximum number of infos to be returned, and the second one the starting index in the list. The list is an array of structure. Each structure contains the following fields : Login, ClientName and IPAddress.
		[XmlRpcMethod] SPlayerInfoBanned[] GetBanList(int _Count, int _BeginIndex);

		// Blacklist the player with the specified login. Only available to SuperAdmin.
		[XmlRpcMethod] bool BlackList(string _Login);

		// UnBlackList the player with the specified login. Only available to SuperAdmin.
		[XmlRpcMethod] bool UnBlackList(string _Login);

		// Clean the blacklist of the server. Only available to SuperAdmin.
		[XmlRpcMethod] bool CleanBlackList();

		// Returns the list of player blacklisted. This method take two parameters. The first parameter specifies the maximum number of infos to be returned, and the second one the starting index in the list. The list is an array of structure. Each structure contains the following fields : Login.
		[XmlRpcMethod] SPlayerInfoBlack[] GetBlackList(int _Count, int _BeginIndex);

		// Add the player with the specified login on the guest list. Only available to Admin.
		[XmlRpcMethod] bool AddGuest(string _Login);

		// Remove the player with the specified login from the guest list. Only available to Admin.
		[XmlRpcMethod] bool RemoveGuest(string _Login);

		// Clean the guest list of the server. Only available to Admin.
		[XmlRpcMethod] bool CleanGuestList();

		// Returns the list of player on the guest list. This method take two parameters. The first parameter specifies the maximum number of infos to be returned, and the second one the starting index in the list. The list is an array of structure. Each structure contains the following fields : Login.
		[XmlRpcMethod] SPlayerInfoBlack[] GetGuestList(int _Count, int _BeginIndex);

		// == SERVER PARAMETERS ==============================
		
		// Set a new server name in utf8 format. Only available to Admin.
		[XmlRpcMethod] bool SetServerName(string _ServerName);
		
		// Get the server name in utf8 format.
		[XmlRpcMethod] string GetServerName();

		// Set a new server comment in utf8 format. Only available to Admin.
		[XmlRpcMethod] bool SetServerComment(string _ServerComment);

		// Get the server comment in utf8 format.
		[XmlRpcMethod] string GetServerComment();

		// Set a new password for the server. Only available to Admin.
		[XmlRpcMethod] bool SetServerPassword(string _ServerPassword);

		// Get the server password if called as Admin or Super Admin, else returns if a password is needed or not.
		[XmlRpcMethod] string GetServerPassword();

		// Set a new password for the spectator mode. Only available to Admin.
		[XmlRpcMethod] bool SetServerPasswordForSpectator(string _ServerPassword);

		// Get the password for spectator mode if called as Admin or Super Admin, else returns if a password is needed or not..
		[XmlRpcMethod] string GetServerPasswordForSpectator();

		// Set a new maximum number of players. Only available to Admin. Requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetMaxPlayers(int _MaxPlayer);

		// Get the current and next maximum number of players allowed on server. The struct returned contains two fields CurrentValue and NextValue.
		[XmlRpcMethod] SValuePair GetMaxPlayers();

		// Set a new maximum number of Spectators. Only available to Admin. Requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetMaxSpectators(int _MaxPlayer);

		// Get the current and next maximum number of Spectators allowed on server. The struct returned contains two fields CurrentValue and NextValue.
		[XmlRpcMethod] SValuePair GetMaxSpectators();

		// Enable or disable peer-to-peer upload from server. Only available to Admin.
		[XmlRpcMethod] bool EnableP2PUpload(bool _Enable);

		// Returns if the peer-to-peer upload from server is enabled.
		[XmlRpcMethod] bool IsP2PUpload();

		// Enable or disable peer-to-peer download for server. Only available to Admin.
		[XmlRpcMethod] bool EnableP2PDownload(bool _Enable);

		// Returns if the peer-to-peer download for server is enabled.
		[XmlRpcMethod] bool IsP2PDownload();

		//Allow clients to download challenges from the server. Only available to Admin.
		[XmlRpcMethod] bool AllowChallengeDownload(bool _Allow);

		//Returns if clients can download challenges from the server.
		[XmlRpcMethod] bool IsChallengeDownloadAllowed();

		// Set a new ladder mode between ladder inactive (0), normal mode (1), and forced ladder (2). Only available to Admin. Requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetLadderMode(int _Mode);

		// Get the current and next ladder mode on server. The struct returned contains two fields CurrentValue and NextValue.
		[XmlRpcMethod] SValuePair GetLadderMode();

		// Set the network vehicle quality to Fast (0); or High (1);. Only available to Admin. Requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetVehicleNetQuality(int _Quality);

		// Get the current and next network vehicle quality on server. The struct returned contains two fields CurrentValue and NextValue.
		[XmlRpcMethod] SValuePair GetVehicleNetQuality();

		//Set a new timeout for waiting for votes. A zero value disables callvote. Only available to Admin. Requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetCallVoteTimeOut(int _TimeOutMs);

		//Get the current and next timeout for waiting for votes. The struct returned contains two fields 'CurrentValue' and 'NextValue'.
		[XmlRpcMethod] SValuePair GetCallVoteTimeOut();

		//Set a new ratio for accepting callvote. Must lie between 0 and 1. Only available to Admin.
		[XmlRpcMethod] bool SetCallVoteRatio(double _Ratio);

		//Get the current ratio for accepting callvote. This value lies between 0 and 1.
		[XmlRpcMethod] double GetCallVoteRatio();

		// Set new server options using the struct passed as parameters. This struct must contain the following fields : Name, Comment, Password, PasswordForSpectator, NextMaxPlayers, NextMaxSpectators, IsP2PUpload, IsP2PDownload, NextLadderMode, NextVehicleNetQuality, NextCallVoteTimeOut, CallVoteRatio and AllowChallengeDownload. Only available to Admin. A change of NextMaxPlayers, NextMaxSpectators, NextLadderMode, NextVehicleNetQuality or NextCallVoteTimeOut requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetServerOptions(SServerOptions _Options);

		// Returns a struct containing the server options i.d. Name, Comment, Password, PasswordForSpectator, CurrentMaxPlayers, NextMaxPlayers, CurrentMaxSpectators, NextMaxSpectators, IsP2PUpload, IsP2PDownload, CurrentLadderMode, NextLadderMode, CurrentVehicleNetQuality, NextVehicleNetQuality, CurrentCallVoteTimeOut, NextCallVoteTimeOut, CallVoteRatio and AllowChallengeDownload.
		[XmlRpcMethod] SServerOptions GetServerOptions();

		// Load the black list file with the specified file name. Only available to SuperAdmin.
		[XmlRpcMethod] bool LoadBlackList(string _FileName);

		// Save the black list in the file with specified file name. Only available to SuperAdmin.
		[XmlRpcMethod] bool SaveBlackList(string _FileName);

		// Load the guest list file with the specified file name. Only available to Admin.
		[XmlRpcMethod] bool LoadGuestList(string _FileName);

		// Save the guest list in the file with specified file name. Only available to SuperAdmin.
		[XmlRpcMethod] bool SaveGuestList(string _FileName);

		// Returns the last error message for an internet connection. Only available to SuperAdmin.
		[XmlRpcMethod] string GetLastConnectionErrorMessage();


		// == GAME PARAMETERS ==============================

		// Set new game settings using the struct passed as parameters. This struct must contain the following fields : GameMode, RoundsPointsLimit, RoundsUseNewRules, TimeAttackLimit, TimeAttackSynchStartPeriod, TeamPointsLimit, TeamMaxPoints, TeamUseNewRules, LapsNbLaps, LapsTimeLimit and ChatTime. Only available to Admin. Requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetGameInfos(SGameInfo _Infos);

		// Returns a struct containing the current game settings i.d. GameMode, RoundsPointsLimit, RoundsUseNewRules, TimeAttackLimit, TimeAttackSynchStartPeriod, TeamPointsLimit, TeamMaxPoints, TeamUseNewRules, LapsNbLaps, LapsTimeLimit, ChatTime and NbChallenge.
		[XmlRpcMethod] SGameInfo GetCurrentGameInfo();

		// Returns a struct containing the game settings for the next challenge i.d. GameMode, RoundsPointsLimit, RoundsUseNewRules, TimeAttackLimit, TimeAttackSynchStartPeriod, TeamPointsLimit, TeamMaxPoints, TeamUseNewRules, LapsNbLaps, LapsTimeLimit, ChatTime and NbChallenge.
		[XmlRpcMethod] SGameInfo GetNextGameInfo();

		// Returns a struct containing two other structures, the first containing the current game settings and the second the game settings for next challenge. The first structure is named CurrentGameInfos and the second NextGameInfos.
		[XmlRpcMethod] SFullGameInfo GetGameInfos();

		// Set a new chat time value in milliseconds. Only available to Admin. Requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetChatTime(int _Time);

		// Get the current and next chat time. The struct returned contains two fields CurrentValue and NextValue.
		[XmlRpcMethod] SValuePair GetChatTime();

		// Set a new game mode between Rounds (0);, TimeAttack (1);, Team (2);, Laps (3); and Stunts (4);. Only available to Admin. Requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetGameMode(int _GameMode);

		// Get the current game mode.
		[XmlRpcMethod] int GetGameMode();

		// Set a new time limit for time attack mode. Only available to Admin. Requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetTimeAttackLimit(int _Limit);

		// Get the current and next time limit for time attack mode. The struct returned contains two fields CurrentValue and NextValue.
		[XmlRpcMethod] SValuePair GetTimeAttackLimit();

		// Set a new synchronised start period for time attack mode. Only available to Admin. Requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetTimeAttackSynchStartPeriod(int _Period);

		// Get the current and synchronised start period for time attack mode. The struct returned contains two fields CurrentValue and NextValue.
		[XmlRpcMethod] SValuePair GetTimeAttackSynchStartPeriod();

		// Set a new time limit for laps mode. Only available to Admin. Requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetLapsTimeLimit(int _Limit);

		// Get the current and next time limit for laps mode. The struct returned contains two fields CurrentValue and NextValue.
		[XmlRpcMethod] SValuePair GetLapsTimeLimit();

		// Set a new number of laps for laps mode. Only available to Admin. Requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetNbLaps(int _NbLaps);

		// Get the current and next number of laps for laps mode. The struct returned contains two fields CurrentValue and NextValue.
		[XmlRpcMethod] SValuePair GetNbLaps();

		// Set a new points limit for rounds mode. Only available to Admin. Requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetRoundPointsLimit(int _Limit);

		// Get the current and next points limit for rounds mode. The struct returned contains two fields CurrentValue and NextValue.
		[XmlRpcMethod] SValuePair GetRoundPointsLimit();

		// Set if new rules are used for rounds mode. Only available to Admin. Requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetUseNewRulesRound(bool _Use);

		// Get if the new rules are used for rounds mode (Current and next values);. The struct returned contains two fields CurrentValue and NextValue.
		[XmlRpcMethod] SValuePair GetUseNewRulesRound();

		// Set a new points limit for team mode. Only available to Admin. Requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetTeamPointsLimit(int _Limit);

		// Get the current and next points limit for team mode. The struct returned contains two fields CurrentValue and NextValue.
		[XmlRpcMethod] SValuePair GetTeamPointsLimit();

		// Set a new number of maximum points per round for team mode. Only available to Admin. Requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetMaxPointsTeam(int _Points);

		// Get the current and next number of maximum points per round for team mode. The struct returned contains two fields CurrentValue and NextValue.
		[XmlRpcMethod] SValuePair GetMaxPointsTeam();

		// Set if new rules are used for team mode. Only available to Admin. Requires a challenge restart to be taken into account.
		[XmlRpcMethod] bool SetUseNewRulesTeam(bool _Use);

		// Get if the new rules are used for team mode (Current and next values);. The struct returned contains two fields CurrentValue and NextValue.
		[XmlRpcMethod] SValuePair GetUseNewRulesTeam();


		// == CHALLENGES CONTROL ==============================

		// Returns the current challenge index in the selection.
		[XmlRpcMethod] int GetCurrentChallengeIndex();

		// Returns a struct containing the infos for the current challenge. The struct contains the following fields : Name, UId, FileName, Author, Environnement, Mood, BronzeTime, SilverTime, GoldTime, AuthorTime, CopperPrice and LapRace.
		[XmlRpcMethod] SChallengeInfo GetCurrentChallengeInfo();

		// Returns a struct containing the infos for the challenge with the specified filename. The struct contains the following fields : Name, UId, FileName, Author, Environnement, Mood, BronzeTime, SilverTime, GoldTime, AuthorTime, CopperPrice and LapRace.
		[XmlRpcMethod] SChallengeInfo GetChallengeInfo(string _Filename);

		// Returns a list of challenges among the current selection of the server. This method take two parameters. The first parameter specifies the maximum number of infos to be returned, and the second one the starting index in the selection. The list is an array of structure. Each structure contains the following field : Name, UId, FileName, Environnement, Mood, GoldTime and CopperPrice.
		[XmlRpcMethod] SChallengeQuickInfo[] GetChallengeList(int _MaxCount, int _FirstIndex);

		// Add the challenge with the specified filename at the end of the current selection. Only available to Admin.
		[XmlRpcMethod] bool AddChallenge(string _Filename);

		// Add the list of challenges with the specified filename at the end of the current selection. Only available to Admin.
		[XmlRpcMethod] int AddChallengeList(string[] _Filenames);

		// Remove the challenge with the specified filename from the current selection. Only available to Admin.
		[XmlRpcMethod] bool RemoveChallenge(string _Filename);

		// Remove the list of challenges with the specified filename from the current selection. Only available to Admin.
		[XmlRpcMethod] int RemoveChallengeList(string[] _Filenames);

		// Insert the challenge with the specified filename after the current challenge. Only available to Admin.
		[XmlRpcMethod] bool InsertChallenge(string _Filename);

		// Insert the list challenges with the specified filename after the current challenge. The list of challenge to insert is an array of strings. Only available to Admin.
		[XmlRpcMethod] int InsertChallengeList(string[] _Filenames);

		// Set as next challenge the one with the specified filename, if it is present in the selection. Only available to Admin.
		[XmlRpcMethod] bool ChooseNextChallenge(string _Filename);

		// Set as next challenge the list of challenges with the specified filenames, if they are present in the selection. Only available to Admin.
		[XmlRpcMethod] int ChooseNextChallengeList(string[] _Filenames);

		// Set a list of challenges defined in the playlist with the specified filename as the current selection of the server, and load the gameinfos from the same file. Only available to Admin.
		[XmlRpcMethod] int LoadMatchSettings(string _Filename);

		// Add a list of challenges defined in the playlist with the specified filename at the end of the current selection. Only available to Admin.
		[XmlRpcMethod] int AppendPlaylistFromMatchSettings(string _Filename);

		// Save the current selection of challenge in the playlist with the specified filename, as well as the current gameinfos. Only available to Admin.
		[XmlRpcMethod] int SaveMatchSettings(string _Filename);

		// Insert a list of challenges defined in the playlist with the specified filename after the current challenge. Only available to Admin.
		[XmlRpcMethod] int InsertPlaylistFromMatchSettings(string _Filename);


		// == PLAYERS ==============================

		// Returns the list of players on the server. This method take two parameters. The first parameter specifies the maximum number of infos to be returned, and the second one the starting index in the list. The list is an array of structure. Each structure contains the following field : Login, NickName, PlayerId, TeamId, IsSpectator, IsInOfficialMode and LadderRanking.
		[XmlRpcMethod] SPlayerQuickInfo[] GetPlayerList(int _Count, int _BeginIndex);

		// Returns a struct containing the infos on the player with the specified login. The structure contains the following fields : Login, NickName, IPAddress, ConnectionType, PlayerId, IsSpectator, IsInOfficialMode, HasJoinMatch, a structure named Avatar, an array of structure named Skins and a structure named LadderStats. The structure Avatar and each structure of the array Skins contains two fields FileName and Checksum. The structure LadderStats contains the following fields : Ranking, Score, LastMatchScore, NbrMatchWins, NbrMatchDraws, NbrMatchLosses, NbrTotalPlayers, TeamName, TeamRanking, NbrTotalTeams.
		[XmlRpcMethod] SPlayerInfo GetPlayerInfo(string _Login);

		// Returns the current ranking for the race in progress. This method take two parameters. The first parameter specifies the maximum number of infos to be returned, and the second one the starting index in the ranking. The ranking returned is a list of structure. Each structure contains the following fields : <i>Login</i>, <i>NickName</i>, <i>PlayerId</i>, <i>Rank</i>, <i>BestTime</i>, <i>Score</i>, <i>NbrLapsFinished</i> and <i>LadderScore</i>. it also contains an array <i>BestCheckpoints</i> that contains the checkpoints times for the best race.
		[XmlRpcMethod] int GetCurrentRanking(int _Count, int _StartIndex);

		// == SERVER CONTROL ==============================

		// Start a server on lan, using the current configuration. Only available to SuperAdmin.
		[XmlRpcMethod] bool StartServerLan();

		// Start a server on internet using the 'Login' and 'Password' specified in the struct passed as parameters. Only available to SuperAdmin.
		[XmlRpcMethod] bool StartServerInternet(SStartServerParamInternet _Param);

		// Returns the current status of the server.
		[XmlRpcMethod] SStatus GetStatus();

		// Restarts the challenge. Only available to Admin.
		[XmlRpcMethod] bool ChallengeRestart();

		// Switch to next challenge. Only available to Admin.
		[XmlRpcMethod] bool NextChallenge();

		// Stop the server. Only available to SuperAdmin.
		[XmlRpcMethod] bool StopServer();

		// In Rounds or Laps mode, force the end of round without waiting for all players to giveup/finish. Only available to Admin.
		[XmlRpcMethod] bool ForceEndRound();

		// Quit the application. Only available to SuperAdmin.
		[XmlRpcMethod] bool QuitGame();


		// == OTHERS ==============================

		// Returns a struct containing the networks stats of the server. The structure contains the following fields : Uptime, NbrConnection, MeanConnectionTime, MeanNbrPlayer, RecvNetRate, SendNetRate, TotalReceivingSize, TotalSendingSize and an array of structure named PlayerNetInfos. Each structure of the array PlayerNetInfos contains the following fields : Login, IPAddress, LastTransferTime, DeltaBetweenTwoLastNetState, PacketLossRate. Only available to SuperAdmin.
		[XmlRpcMethod] SNetworkStats GetNetworkStats();

		// Returns a replay containing the data needed to validate the current best time of the player.
		//[XmlRpcMethod] byte[] GetValidationReplay(string _Login);

		// Returns the path of the dedicated server application. Only available to Admin.
		[XmlRpcMethod] string GetBaseDirectory();

		// Returns the path of the game datas directory. Only available to Admin.
		[XmlRpcMethod] string GameDataDirectory();

		// Returns the path of the tracks directory. Only available to Admin.
		[XmlRpcMethod] string GetTracksDirectory();

		// Returns the path of the skins directory. Only available to Admin.
		[XmlRpcMethod] string GetSkinsDirectory();

		// Allow the GameServer to call you back.
		[XmlRpcMethod] bool EnableCallbacks(bool _Enabled);

		// Write the data to the specified file. The filename is relative to the Tracks path. Only available to Admin.
		[XmlRpcMethod] bool WriteFile(string _FileName, byte[] _Data);
	}	
}
